﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace QuickStock.Model
{
    public class BL
    {

        public static bool LoadArticolo_DDT(string codiceDDT, int idArticoloOrdine, int quantita, string note, int idMagazzino) 
        {
            try 
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext()) 
                {
                    ARTICOLI_DDT_CARICO articoloDDT = new ARTICOLI_DDT_CARICO();
                    articoloDDT.CODICE_DDT_CARICO = codiceDDT;
                    articoloDDT.ID_ARTICOLO_ORDINE_CLIENTE = idArticoloOrdine;
                    articoloDDT.QUANTITA = quantita;
                    articoloDDT.NOTE = note;
                    articoloDDT.ID_MAGAZZINO = idMagazzino;

                    quickStockDB.ARTICOLI_DDT_CARICO.InsertOnSubmit(articoloDDT);
                    quickStockDB.SubmitChanges();
                }
            }
            catch { return false; }

            return true;
        }

        public static bool LoadArticolo_DDT_OrderLess(string codiceDDT, string codiceArticolo, int quantita, string note, int idMagazzino)
        {
            try
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
                {
                    ARTICOLI_DDT_CARICO articoloDDT = new ARTICOLI_DDT_CARICO();
                    articoloDDT.CODICE_DDT_CARICO = codiceDDT;
                    articoloDDT.CODICE_ARTICOLO = codiceArticolo;
                    articoloDDT.QUANTITA = quantita;
                    articoloDDT.ID_MAGAZZINO = idMagazzino;
                    articoloDDT.NOTE = note;

                    quickStockDB.ARTICOLI_DDT_CARICO.InsertOnSubmit(articoloDDT);
                    quickStockDB.SubmitChanges();
                }
            }
            catch { return false; }

            return true;
        }

        public static DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable GetArticoliOrdinati_ByCodice(string codice) 
        {
            DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable tableArticoli = new DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable();

            if (String.IsNullOrEmpty(codice)) 
            {
                return tableArticoli;
            }

            if (codice.Equals("%"))
            {
                codice = String.Empty;
            }


            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_CLIENTE
                                where (a.CODICE_ARTICOLO_UNICO.StartsWith(codice)
                                    && a.ID_STATO_ARTICOLO_ORDINE_CLIENTE == 1)
                                select a);

                foreach (ARTICOLI_ORDINE_CLIENTE articolo in articoli)
                {
                    try
                    {
                        ARTICOLI_ORDINE_CLIENTE _articolo = articolo;

                        DSOrdini.ARTICOLI_ORDINE_CLIENTERow rowArt = (DSOrdini.ARTICOLI_ORDINE_CLIENTERow)tableArticoli.NewRow();
                        rowArt.ID_ARTICOLO_ORDINE_CLIENTE = articolo.ID_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.QUANTITA_ARTICOLO_ORDINE_CLIENTE = articolo.QUANTITA_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.RAGSOC_CLIENTE = articolo.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                        rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                        if (articolo.QUANTITA_EVASA.HasValue)
                        {
                            rowArt.QUANTITA_EVASA = (int)articolo.QUANTITA_EVASA;
                        }

                        rowArt.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.ID_FORNITORE = (int)articolo.ID_FORNITORE;
                        rowArt.CODICE_ARTICOLO_UNICO = articolo.CODICE_ARTICOLO_UNICO;
                        rowArt.DATA_ORDINE_CLIENTE = articolo.ORDINE_CLIENTE.DATA_ORDINE_CLIENTE.ToShortDateString();
                        rowArt.CODICE_ORDINE_FORNITORE = articolo.ARTICOLI_ORDINE_FORNITORE.First().CODICE_ORDINE;
                        rowArt.FORNITORE = articolo.ARTICOLI_ORDINE_FORNITORE.First().FORNITORE.RAGIONE_SOCIALE_FORNITORE;
                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }

            return tableArticoli;
        }

        public static DSOrdini.ARTICOLI_DDT_CARICODataTable GetArticoliDDT_Carico_ByCodiceDDT(string codice) 
        {
            DSOrdini.ARTICOLI_DDT_CARICODataTable tableArticoli = new DSOrdini.ARTICOLI_DDT_CARICODataTable();

            if (String.IsNullOrEmpty(codice))
            {
                return tableArticoli;
            }

            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_DDT_CARICO
                                where (a.CODICE_DDT_CARICO.Equals(codice))
                                select a);

                foreach (ARTICOLI_DDT_CARICO articolo in articoli)
                {
                    try
                    {
                        DSOrdini.ARTICOLI_DDT_CARICORow rowArt = (DSOrdini.ARTICOLI_DDT_CARICORow)tableArticoli.NewRow();

                        rowArt.ID_ARTICOLI_DDT_CARICO = articolo.ID_ARTICOLI_DDT_CARICO;

                        try { 
                                rowArt.CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                                rowArt.CLIENTE_TITLE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE; 
                            }
                        catch { }

                        try { rowArt.CODICE_ORDINE = articolo.ARTICOLI_ORDINE_CLIENTE.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE; }
                        catch { }

                        try { rowArt.CODICE_ARTICOLO_ORDINATO = articolo.ARTICOLI_ORDINE_CLIENTE.CODICE_ARTICOLO_UNICO; }
                        catch { }

                        try { rowArt.CODICE_ARTICOLO = articolo.CODICE_ARTICOLO; }
                        catch { }
                        
                        try { rowArt.DATA_ORDINE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.DATA_ORDINE_CLIENTE.ToShortDateString(); }
                        catch { }
                        
                        try { rowArt.QUANTITA = articolo.QUANTITA; }
                        catch { }
                        
                        try { rowArt.NOTE = articolo.NOTE; }
                        catch { }

                        rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                        rowArt.NOME_MAGAZZINO = articolo.MAGAZZINO.NOME_MAGAZZINO;
                        
                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }

            return tableArticoli;
 
        }

        public static DSOrdini.ARTICOLI_DDT_CARICODataTable GetArticoliDDT_Carico_NotAssigned(string codiceArticolo) 
        {
            DSOrdini.ARTICOLI_DDT_CARICODataTable tableArticoli = new DSOrdini.ARTICOLI_DDT_CARICODataTable();

            if (String.IsNullOrEmpty(codiceArticolo))
            {
                return tableArticoli;
            }

            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_DDT_CARICO
                                where (a.ID_ARTICOLO_ORDINE_CLIENTE == null && a.ARTICOLO.CODICE_ARTICOLO.StartsWith(codiceArticolo))
                                select a);

                foreach (ARTICOLI_DDT_CARICO articolo in articoli)
                {
                    try
                    {
                        DSOrdini.ARTICOLI_DDT_CARICORow rowArt = (DSOrdini.ARTICOLI_DDT_CARICORow)tableArticoli.NewRow();

                        rowArt.ID_ARTICOLI_DDT_CARICO = articolo.ID_ARTICOLI_DDT_CARICO;

                        try
                        {
                            rowArt.CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                            rowArt.CLIENTE_TITLE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                        }
                        catch { }

                        try { rowArt.CODICE_ORDINE = articolo.ARTICOLI_ORDINE_CLIENTE.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE; }
                        catch { }

                        try { rowArt.CODICE_ARTICOLO_ORDINATO = articolo.ARTICOLI_ORDINE_CLIENTE.CODICE_ARTICOLO_UNICO; }
                        catch { }

                        try { rowArt.CODICE_ARTICOLO = articolo.CODICE_ARTICOLO; }
                        catch { }

                        try { rowArt.DATA_ORDINE = articolo.ARTICOLI_ORDINE_CLIENTE.ORDINE_CLIENTE.DATA_ORDINE_CLIENTE.ToShortDateString(); }
                        catch { }

                        try { rowArt.QUANTITA = articolo.QUANTITA; }
                        catch { }

                        try { rowArt.NOTE = articolo.NOTE; }
                        catch { }

                        rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                        rowArt.NOME_MAGAZZINO = articolo.MAGAZZINO.NOME_MAGAZZINO;

                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }

            return tableArticoli;

        }

        public static DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable GetArticoliOrdineFornitore_ByCodiceOrdine(string codice) 
        {
            DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable tableArticoli = new DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable();
            if (String.IsNullOrEmpty(codice))
            {
                return tableArticoli;
            }

            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_FORNITORE
                                where (a.CODICE_ORDINE.Equals(codice))
                                select a);

                foreach (ARTICOLI_ORDINE_FORNITORE articolo in articoli)
                {
                    try
                    {
                        DSOrdini.ARTICOLI_ORDINE_FORNITORERow rowArt = (DSOrdini.ARTICOLI_ORDINE_FORNITORERow)tableArticoli.NewRow();

                        rowArt.ID_ARTICOLO_ORDINE_FORNITORE = articolo.ID_ARTICOLO_ORDINE_FORNITORE;

                        try{ rowArt.STATO = articolo.STATO_ARTICOLO_ORDINE.NOME_STATO_ARTICOLO_ORDINE; }
                        catch { }

                        try { rowArt.QUANTITA_ORDINATA = articolo.QUANTITA_ARTICOLO_ORDINE_FORNITORE; }
                        catch { }

                        try { rowArt.QUANTITA_EVASA = (int)articolo.QUANTITA_EVASA; }
                        catch { }

                        try 
                        { 
                            rowArt.CODICE_ARTICOLO = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_FORNITORE;
                            rowArt.CODICE_ARTICOLO_TITLE = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_FORNITORE;
                            rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                            rowArt.PRODUTTORE = articolo.PRODUTTORE.NOME_PRODUTTORE;
                        }
                        catch { }

                        try { rowArt.CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE_FORNITORE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE; }
                        catch { }

                        try { rowArt.CODICE_ORDINE_CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE_FORNITORE.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE; }
                        catch { }

                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }
            return tableArticoli;
        }

        public static DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable GetArticoliOrdineFornitore(int? idFornitore, string codiceOrdine, int? idProduttore, int? idStato, string codiceArticolo, string dataOrdineFornitore_DA, string dataOrdineFornitore_A) 
        {
            DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable tableArticoli = new DSOrdini.ARTICOLI_ORDINE_FORNITOREDataTable();
            if (codiceOrdine == null) { codiceOrdine = String.Empty; }

            if (codiceArticolo == null) { codiceArticolo = String.Empty; }

            if (idFornitore == null) { idFornitore = -1; }

            if (idProduttore == null) { idProduttore = -1; }

            if (idStato == null) { idStato = -1; }

            DateTime? dataOrdine_DA = null;
            DateTime? dataOrdine_A = null;

            try { dataOrdine_DA = DateTime.Parse(dataOrdineFornitore_DA); }catch { }
            try { dataOrdine_A = DateTime.Parse(dataOrdineFornitore_A); }catch { }

            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_FORNITORE
                                where (
                                    a.CODICE_ORDINE.StartsWith(codiceOrdine)
                                    &&
                                    (a.ID_FORNITORE == (int)idFornitore || idFornitore == -1)
                                    &&
                                    (a.ARTICOLO.ID_PRODUTTORE_ARTICOLO == (int)idProduttore || idProduttore == -1)
                                    &&
                                    (a.ID_STATO_ARTICOLO_ORDINE_FORNITORE == (int)idStato || idStato == -1)
                                    &&
                                    a.CODICE_ARTICOLO_ARTICOLO_ORDINE_FORNITORE.StartsWith(codiceArticolo)
                                    &&
                                    (a.ORDINE_FORNITORE.DATA_ORDINE_FORNITORE >= dataOrdine_DA || dataOrdine_DA == null)
                                    &&
                                    (a.ORDINE_FORNITORE.DATA_ORDINE_FORNITORE <= dataOrdine_A || dataOrdine_A == null)
                                )
                                select a);

                foreach (ARTICOLI_ORDINE_FORNITORE articolo in articoli)
                {
                    try
                    {
                        DSOrdini.ARTICOLI_ORDINE_FORNITORERow rowArt = (DSOrdini.ARTICOLI_ORDINE_FORNITORERow)tableArticoli.NewRow();

                        rowArt.ID_ARTICOLO_ORDINE_FORNITORE = articolo.ID_ARTICOLO_ORDINE_FORNITORE;

                        try { rowArt.STATO = articolo.STATO_ARTICOLO_ORDINE.NOME_STATO_ARTICOLO_ORDINE; }
                        catch { }

                        try { rowArt.QUANTITA_ORDINATA = articolo.QUANTITA_ARTICOLO_ORDINE_FORNITORE; }
                        catch { }

                        try { rowArt.QUANTITA_EVASA = (int)articolo.QUANTITA_EVASA; }
                        catch { }

                        try
                        {
                            rowArt.CODICE_ARTICOLO = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_FORNITORE;
                            rowArt.CODICE_ARTICOLO_TITLE = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_FORNITORE;
                            rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                            rowArt.PRODUTTORE = articolo.PRODUTTORE.NOME_PRODUTTORE;
                            rowArt.FORNITORE = articolo.FORNITORE.RAGIONE_SOCIALE_FORNITORE;
                            rowArt.CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE_FORNITORE.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                            rowArt.CODICE_ORDINE_CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE_FORNITORE.ORDINE_CLIENTE.CODICE_ORDINE_CLIENTE;
                            rowArt.DATA_ORDINE_FORNITORE = articolo.ORDINE_FORNITORE.DATA_ORDINE_FORNITORE.ToShortDateString();
                            rowArt.DATA_ORDINE_CLIENTE = articolo.ARTICOLI_ORDINE_CLIENTE_FORNITORE.ORDINE_CLIENTE.DATA_ORDINE_CLIENTE.ToShortDateString();
                        }
                        catch { }

                        try { rowArt.CODICE_ORDINE_FORNITORE = articolo.CODICE_ORDINE; }
                        catch { }

                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }
            return tableArticoli; 
        }

        public static DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable GetArticoliOrdineCliente_ByCodiceOrdine(string codice) 
        {
            DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable tableArticoli = new DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable();

            if (String.IsNullOrEmpty(codice))
            {
                return tableArticoli;
            }

            if (codice.Equals("%"))
            {
                codice = String.Empty;
            }


            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_CLIENTE
                                where (a.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE.Equals(codice))
                                select a);

                foreach (ARTICOLI_ORDINE_CLIENTE articolo in articoli)
                {
                    try
                    {
                        ARTICOLI_ORDINE_CLIENTE _articolo = articolo;

                        DSOrdini.ARTICOLI_ORDINE_CLIENTERow rowArt = (DSOrdini.ARTICOLI_ORDINE_CLIENTERow)tableArticoli.NewRow();
                        rowArt.ID_ARTICOLO_ORDINE_CLIENTE = articolo.ID_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.QUANTITA_ARTICOLO_ORDINE_CLIENTE = articolo.QUANTITA_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.RAGSOC_CLIENTE = articolo.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                        rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                        
                        if (articolo.QUANTITA_EVASA.HasValue)
                        {
                            rowArt.QUANTITA_EVASA = (int)articolo.QUANTITA_EVASA;
                        }
                        else 
                        {
                            rowArt.QUANTITA_EVASA = 0;
                        }

                        rowArt.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE;
                        try
                        {
                            rowArt.ID_FORNITORE = (int)articolo.ID_FORNITORE;
                            rowArt.FORNITORE = articolo.FORNITORE.RAGIONE_SOCIALE_FORNITORE;
                        }
                        catch { }
                        
                        rowArt.CODICE_ARTICOLO_UNICO = articolo.CODICE_ARTICOLO_UNICO;
                        rowArt.DESCRIZIONE_STATO = articolo.STATO_ARTICOLO_ORDINE.NOME_STATO_ARTICOLO_ORDINE;

                        try 
                        {
                            rowArt.NOTE_MAGAZZINO = articolo.ARTICOLI_DDT_CARICO.First().MAGAZZINO.NOME_MAGAZZINO;
                            if (!articolo.ARTICOLI_DDT_CARICO.First().NOTE.Equals(String.Empty)) 
                            {
                                rowArt.NOTE_MAGAZZINO = rowArt.NOTE_MAGAZZINO + " [ " + articolo.ARTICOLI_DDT_CARICO.First().NOTE + "]";
                            }
                        }
                        catch { }


                        // memorizza i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch (Exception ex) 
                    {
                        string s = ex.Message; 
                    }
                }
            }

            return tableArticoli;
        }

        public static DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable GetArticoliOrdineCliente(int? idFornitore, string codiceOrdine, int? idProduttore, int? idStato, string codiceArticolo) 
        {
            DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable tableArticoli = new DSOrdini.ARTICOLI_ORDINE_CLIENTEDataTable();

            if (codiceOrdine == null) { codiceOrdine = String.Empty; }

            if (codiceArticolo == null) { codiceArticolo = String.Empty; }

            if (idFornitore == null){idFornitore = -1;}

            if (idProduttore == null){idProduttore = -1;}

            if (idStato == null) { idStato = -1; }

            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_CLIENTE
                                where (
                                    a.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE.StartsWith(codiceOrdine)
                                    &&
                                    (a.ID_FORNITORE == (int)idFornitore || idFornitore == -1)
                                    &&
                                    (a.ARTICOLO.ID_PRODUTTORE_ARTICOLO == (int)idProduttore || idProduttore == -1)
                                    &&
                                    (a.ID_STATO_ARTICOLO_ORDINE_CLIENTE == (int)idStato || idStato == -1)
                                    &&
                                    a.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE.StartsWith(codiceArticolo)
                                )
                                select a);

                foreach (ARTICOLI_ORDINE_CLIENTE articolo in articoli)
                {
                    try
                    {
                        ARTICOLI_ORDINE_CLIENTE _articolo = articolo;

                        DSOrdini.ARTICOLI_ORDINE_CLIENTERow rowArt = (DSOrdini.ARTICOLI_ORDINE_CLIENTERow)tableArticoli.NewRow();
                        rowArt.ID_ARTICOLO_ORDINE_CLIENTE = articolo.ID_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.QUANTITA_ARTICOLO_ORDINE_CLIENTE = articolo.QUANTITA_ARTICOLO_ORDINE_CLIENTE;
                        rowArt.RAGSOC_CLIENTE = articolo.ORDINE_CLIENTE.CLIENTE.RAGIONE_SOCIALE_CLIENTE;
                        rowArt.DESCRIZIONE_ARTICOLO = articolo.ARTICOLO.DESCRIZIONE_ARTICOLO;
                        rowArt.FORNITORE = articolo.FORNITORE.RAGIONE_SOCIALE_FORNITORE;
                        if (articolo.QUANTITA_EVASA.HasValue)
                        {
                            rowArt.QUANTITA_EVASA = (int)articolo.QUANTITA_EVASA;
                        }
                        else
                        {
                            rowArt.QUANTITA_EVASA = 0;
                        }

                        rowArt.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = articolo.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE;
                        try { rowArt.ID_FORNITORE = (int)articolo.ID_FORNITORE; }
                        catch { }
                        rowArt.CODICE_ARTICOLO_UNICO = articolo.CODICE_ARTICOLO_UNICO;
                        rowArt.DESCRIZIONE_STATO = articolo.STATO_ARTICOLO_ORDINE.NOME_STATO_ARTICOLO_ORDINE;

                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }

            return tableArticoli;
        }

        public static DSOrdini.ARTICOLODataTable GetArticoli_ByCodice(string codice) 
        {
            DSOrdini.ARTICOLODataTable tableArticoli = new DSOrdini.ARTICOLODataTable();

            if (String.IsNullOrEmpty(codice))
            {
                return tableArticoli;
            }

            if (codice.Equals("%"))
            {
                codice = String.Empty;
            }


            using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
            {
                var articoli = (from a in quickStockDB.ARTICOLO
                                where (a.CODICE_ARTICOLO.StartsWith(codice))
                                select a);

                foreach (ARTICOLO articolo in articoli)
                {
                    try
                    {
                        DSOrdini.ARTICOLORow rowArt = (DSOrdini.ARTICOLORow)tableArticoli.NewRow();
                        rowArt.CODICE_ARTICOLO = articolo.CODICE_ARTICOLO;
                        rowArt.CATEGORIA_ARTICOLO = articolo.CATEGORIA_ARTICOLO;
                        rowArt.CLASSE_ARTICOLO = articolo.CLASSE_ARTICOLO;
                        rowArt.DESCRIZIONE_ARTICOLO = articolo.DESCRIZIONE_ARTICOLO;

                        rowArt.ID_PRODUTTORE_ARTICOLO = articolo.ID_PRODUTTORE_ARTICOLO;

                        try { rowArt.PESO_ARTICOLO = (double)articolo.PESO_ARTICOLO; }
                        catch { }

                        try { rowArt.PEZZI_PER_SCATOLA_ARTICOLO = (int)articolo.PEZZI_PER_SCATOLA_ARTICOLO; }
                        catch { }

                        try { rowArt.PREZZO_ACQUISTO_ARTICOLO = (double)articolo.PREZZO_ACQUISTO_ARTICOLO; }
                        catch { }

                        rowArt.UNITA_MISURA_ARTICOLO = articolo.UNITA_MISURA_ARTICOLO;
                        rowArt.TIPO_ARTICOLO = articolo.TIPO_ARTICOLO;

                        // legge i dati dell'articolo
                        tableArticoli.Rows.Add(rowArt);
                    }
                    catch { }
                }
            }

            return tableArticoli;
        }

        public static bool Insert_Articolo_OrdineCliente(string codiceArticolo, int quantitaOrdinata, int id_Fornitore, string codiceOrdine)
        {
            try 
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext()) 
                {
                    ARTICOLI_ORDINE_CLIENTE articoloOrdine = new ARTICOLI_ORDINE_CLIENTE();
                    articoloOrdine.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE = codiceArticolo;
                    articoloOrdine.QUANTITA_ARTICOLO_ORDINE_CLIENTE = quantitaOrdinata;
                    articoloOrdine.QUANTITA_EVASA = 0;
                    articoloOrdine.ID_FORNITORE = id_Fornitore;
                    articoloOrdine.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = codiceOrdine;
                    articoloOrdine.ID_STATO_ARTICOLO_ORDINE_CLIENTE = 1;

                    quickStockDB.ARTICOLI_ORDINE_CLIENTE.InsertOnSubmit(articoloOrdine);
                    quickStockDB.SubmitChanges();
                }
            }
            catch { return false; }

            return true;

        }

        public static bool Insert_Articolo_OrdineCliente_ByMagazzino(int idArticoloDDT_CARICO, int quantitaOrdinata, string codiceOrdine)
        {
            try
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
                {

                    var articoloDDT_CARICO = (from o in quickStockDB.ARTICOLI_DDT_CARICO
                                              where (
                                                  o.ID_ARTICOLI_DDT_CARICO == idArticoloDDT_CARICO
                                              )
                                              select o).First();

                    // inserisco articolo in ordine
                    ARTICOLI_ORDINE_CLIENTE articoloOrdine = new ARTICOLI_ORDINE_CLIENTE();
                    articoloOrdine.CODICE_ARTICOLO_ARTICOLO_ORDINE_CLIENTE = articoloDDT_CARICO.ARTICOLO.CODICE_ARTICOLO;
                    articoloOrdine.QUANTITA_ARTICOLO_ORDINE_CLIENTE = quantitaOrdinata;
                    articoloOrdine.QUANTITA_EVASA = quantitaOrdinata;
                    articoloOrdine.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = codiceOrdine;
                    articoloOrdine.ID_STATO_ARTICOLO_ORDINE_CLIENTE = 2;

                    quickStockDB.ARTICOLI_ORDINE_CLIENTE.InsertOnSubmit(articoloOrdine);
                    quickStockDB.SubmitChanges();

                    // aggiorno articolo DDT
                    articoloDDT_CARICO.ID_ARTICOLO_ORDINE_CLIENTE = articoloOrdine.ID_ARTICOLO_ORDINE_CLIENTE;                    

                    // aggiotno giacenza in magazzino
                    if (articoloDDT_CARICO.QUANTITA > quantitaOrdinata) 
                    {
                        ARTICOLI_DDT_CARICO articoloDDT = new ARTICOLI_DDT_CARICO();
                        articoloDDT.CODICE_DDT_CARICO = articoloDDT_CARICO.CODICE_DDT_CARICO;
                        articoloDDT.CODICE_ARTICOLO = articoloDDT_CARICO.ARTICOLO.CODICE_ARTICOLO;
                        articoloDDT.QUANTITA = (articoloDDT_CARICO.QUANTITA - quantitaOrdinata);
                        articoloDDT.ID_MAGAZZINO = (int)articoloDDT_CARICO.ID_MAGAZZINO;
                        articoloDDT.NOTE = articoloDDT_CARICO.NOTE;

                        quickStockDB.ARTICOLI_DDT_CARICO.InsertOnSubmit(articoloDDT);
                    }

                    articoloDDT_CARICO.QUANTITA = quantitaOrdinata;
                    quickStockDB.SubmitChanges();

                }
            }
            catch(Exception ex) { return false; }

            return true;

        }
        
        public static DSOrdini.ORDINE_CLIENTEDataTable GetOrdineCliente_ByCodice(string codiceOrdine) 
        {
            DSOrdini.ORDINE_CLIENTEDataTable tableOrdine = new DSOrdini.ORDINE_CLIENTEDataTable();

            try 
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext()) 
                {
                    var ordine = (from o in quickStockDB.ORDINE_CLIENTE
                                    where (
                                        o.CODICE_ORDINE_CLIENTE.Equals(codiceOrdine)
                                    )
                                    select o).First();

                    if (ordine != null) 
                    {
                        DSOrdini.ORDINE_CLIENTERow rowOrdine = (DSOrdini.ORDINE_CLIENTERow)tableOrdine.NewRow();
                        rowOrdine.CODICE_ORDINE_CLIENTE = ordine.CODICE_ORDINE_CLIENTE;
                        rowOrdine.DATA_ORDINE_CLIENTE = ordine.DATA_ORDINE_CLIENTE;
                        try { rowOrdine.DATA_ULTIMA_MODIFICA = (DateTime)ordine.DATA_ULTIMA_MODIFICA; } catch { }
                        rowOrdine.ID_CLIENTE_ORDINE_CLIENTE = ordine.ID_CLIENTE_ORDINE_CLIENTE;
                        rowOrdine.ID_STATO_ORDINE = ordine.ID_STATO_ORDINE;
                        rowOrdine.ID_UTENTE_ULTIMA_MODIFICA = ordine.ID_UTENTE_ULTIMA_MODIFICA;
                        rowOrdine.NOTE_ORDINE_CLIENTE = ordine.NOTE_ORDINE_CLIENTE;

                        tableOrdine.Rows.Add(rowOrdine);
                    }
                }
            }
            catch { }
            return tableOrdine;
        }

        public static bool Update_OrdineCliente(string codiceOrdine, string codiceOrdine_OLD, string note, DateTime dataOrdine, int idCliente, int idStato, int idUtente) 
        {
            try
            {
                using (QuickStockDataContext quickStockDB = new QuickStockDataContext())
                {
                    if (!codiceOrdine.Equals(codiceOrdine_OLD))
                    {
                        // creazione nuovo ordine
                        ORDINE_CLIENTE ordineCliente = new ORDINE_CLIENTE();
                        ordineCliente.CODICE_ORDINE_CLIENTE = codiceOrdine;
                        ordineCliente.DATA_ORDINE_CLIENTE = dataOrdine;
                        ordineCliente.ID_STATO_ORDINE = idStato;
                        ordineCliente.ID_CLIENTE_ORDINE_CLIENTE = idCliente;
                        ordineCliente.ID_UTENTE_ULTIMA_MODIFICA = idUtente;
                        ordineCliente.NOTE_ORDINE_CLIENTE = note;
                        quickStockDB.ORDINE_CLIENTE.InsertOnSubmit(ordineCliente);
                        quickStockDB.SubmitChanges();


                        // associazione articoli al nuovo codice ordine
                        var articoli = (from a in quickStockDB.ARTICOLI_ORDINE_CLIENTE
                                        where (a.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE.Equals(codiceOrdine_OLD))
                                        select a);

                        foreach (ARTICOLI_ORDINE_CLIENTE articolo in articoli)
                        {
                            articolo.CODICE_ORDINE_CLIENTE_ARTICOLO_ORDINE_CLIENTE = codiceOrdine;
                            quickStockDB.SubmitChanges();
                        }

                        // cancellazione vecchio codice ordine
                        var old_ordine_cliente = (from o in quickStockDB.ORDINE_CLIENTE
                                                  where (
                                                      o.CODICE_ORDINE_CLIENTE.Equals(codiceOrdine_OLD)
                                                  )
                                                  select o).First();

                        quickStockDB.ORDINE_CLIENTE.DeleteOnSubmit(old_ordine_cliente);
                        quickStockDB.SubmitChanges();
                    }
                    else 
                    { 
                        // aggiorno ordine
                        var old_ordine_cliente = (from o in quickStockDB.ORDINE_CLIENTE
                                                  where (
                                                      o.CODICE_ORDINE_CLIENTE.Equals(codiceOrdine_OLD)
                                                  )
                                                  select o).First();

                        old_ordine_cliente.DATA_ORDINE_CLIENTE = dataOrdine;
                        old_ordine_cliente.ID_STATO_ORDINE = idStato;
                        old_ordine_cliente.ID_CLIENTE_ORDINE_CLIENTE = idCliente;
                        old_ordine_cliente.ID_UTENTE_ULTIMA_MODIFICA = idUtente;
                        old_ordine_cliente.NOTE_ORDINE_CLIENTE = note;
                        quickStockDB.SubmitChanges();
                    }

                }
            }
            catch { return false; }
            return true;
        }
    }
}
